<?php 

if ( ! defined( 'ABSPATH' ) ) exit; 
 
/**
  Plugin Name: Z-Payment Gateway
  Plugin URI: http://z-payment.com/docs.php
  Description: Provides a Z-Payment Payment Gateway.
  Version: 1.0.0
  Author: Z-Payment
 */


/* Add a custom payment class to WC
  ------------------------------------------------------------ */
add_action('plugins_loaded', 'woocommerce_zpayment', 0);
function woocommerce_zpayment(){
	if (!class_exists('WC_Payment_Gateway'))
		return; 
	if(class_exists('WC_Zpayment'))
		return;
	
class WC_Zpayment extends WC_Payment_Gateway{
	public function __construct(){
		
		$plugin_dir = plugin_dir_url(__FILE__);

		global $woocommerce;

		$this->id = 'zpayment';
		$this->icon = apply_filters('woocommerce_zpayment_icon', ''.$plugin_dir.'zpayment.png');
		$this->has_fields = false;

		$this->init_form_fields();
		$this->init_settings();

		$this->title = $this->get_option('title');
		$this->ID = $this->get_option('ID');
		$this->MERCH_KEY = $this->get_option('MERCH_KEY');
		$this->INIT_PASS = $this->get_option('INIT_PASS');
		$this->METHOD = $this->get_option('METHOD');
		$this->description = $this->get_option('description');
		$this->instructions = $this->get_option('instructions');
		

		if ($this->debug == 'yes'){
			$this->log = $woocommerce->logger();
		}

		
		add_action('woocommerce_receipt_' . $this->id, array($this, 'receipt_page'));
		add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
		add_action('woocommerce_api_wc_' . $this->id, array($this, 'check_assistant_response'));

	}
	


  /**
  * Форма настроек платежного шлюза 
  *
  * @access public
  * @return void
  */
	function init_form_fields(){
	
	$resulturl=site_url('?wc-api=wc_zpayment&zpayment=callback');
	$successurl=site_url('?wc-api=wc_zpayment&zpayment=success');
	$failurl=site_url('?wc-api=wc_zpayment&zpayment=fail');
	
		$this->form_fields = array(
				'enabled' => array(
					'title' => __('Включить/Выключить', 'woocommerce'),
					'type' => 'checkbox',
					'label' => __('Включен', 'woocommerce'),
					'default' => 'yes'
				),
				'title' => array(
					'title' => __('Название', 'woocommerce'),
					'type' => 'text', 
					'description' => __( 'Это название, которое пользователь видит во время проверки.', 'woocommerce' ), 
					'default' => __('Z-Payment', 'woocommerce')
				),
				'ID' => array(
					'title' => __('Shop ID', 'woocommerce'),
					'type' => 'text',
					'default' => ''
				),
				'MERCH_KEY' => array(
					'title' => __('Merchant key', 'woocommerce'),
					'type' => 'text',
					'default' => ''
				),
				
				'INIT_PASS' => array(
					'title' => __('Initialization Password', 'woocommerce'),
					'type' => 'text',
					'default' => ''
				),

				'description' => array(
					'title' => __( 'Description', 'woocommerce' ),
					'type' => 'textarea',
					'description' => __( 'Описанием метода оплаты которое клиент будет видеть на вашем сайте.', 'woocommerce' ),
					'default' => 'Оплата с помощью Z-Payment.'
				),
				'instructions' => array(
					'title' => __( 'Instructions', 'woocommerce' ),
					'type' => 'textarea',
					'description' => __( 'Инструкции, которые будут добавлены на страницу благодарностей.', 'woocommerce' ),
					'default' => 'Оплата с помощью Z-Payment.'
				),
				
			    'ResultUrl' => array(
		            'title' => __( 'ResultUrl', 'woocommerce' ),
					'type' => 'title',
					'description' => $resulturl,
					'default' => ''
				),	
				
			    'SuccessUrl' => array(
		            'title' => __( 'SuccessUrl', 'woocommerce' ),
					'type' => 'title',
					'description' => $successurl,
					'default' => ''
				),		

			    'FailUrl' => array(
		            'title' => __( 'FailUrl', 'woocommerce' ),
					'type' => 'title',
					'description' => $failurl,
					'default' => ''
				),					

			);
	}

	/**
	* There are no payment fields for sprypay, but we want to show the description if set.
	**/
	function payment_fields(){
		if ($this->description){
			echo wpautop(wptexturize($this->description));
		}
	}
	
	
	
	/**
	* Форма отправки заказа
	**/
	public function generate_form($order_id){
		global $woocommerce;

		$order = new WC_Order( $order_id );

		$action_adr = "https://z-payment.com/merchant.php";

		$out_summ = number_format($order->order_total, 2, '.', '');
		$signature = md5($this->ID.$order_id.$out_summ.$this->INIT_PASS);
		
		$args = array(
				'LMI_PAYEE_PURSE' => $this->ID,
				'LMI_PAYMENT_AMOUNT' => $out_summ,
				'LMI_PAYMENT_NO' => $order_id,
				'ZP_SIGN' => $signature,
				'LMI_PAYMENT_DESC' => 'online_shop_payment'
				
			);

		$args_array = array();

		foreach ($args as $key => $value){
			$args_array[] = '<input type="hidden" name="'.esc_attr($key).'" value="'.esc_attr($value).'" />';
		}

		return
			'<form action="'.esc_url($action_adr).'" method="POST" id="zpayment_payment_form">'."\n".
			implode("\n", $args_array).
			'<input type="submit" class="button alt" id="submit_zpayment_payment_form" value="'.__('Оплатить', 'woocommerce').'" /> <a class="button cancel" href="'.$order->get_cancel_order_url().'">'.__('Отказаться от оплаты & вернуться в корзину', 'woocommerce').'</a>'."\n".
			'</form>';
	}
	
	/**
	 * Process the payment and return the result
	 **/
	function process_payment($order_id){
		$order = new WC_Order($order_id);

		return array(
			'result' => 'success',
			'redirect'	=> add_query_arg('order', $order->id, add_query_arg('key', $order->order_key, get_permalink(woocommerce_get_page_id('pay'))))
		);
	}
	
	/**
	* receipt_page
	**/
	function receipt_page($order){
		echo '<p>'.__('Спасибо за Ваш заказ, пожалуйста, нажмите кнопку ниже, чтобы заплатить.', 'woocommerce').'</p>';
		echo $this->generate_form($order);
	}
	
	/**
	 * Check zpayment IPN validity
	 **/
	 
	function check_zprequest_is_valid($posted){
	    $merch_key=$this->MERCH_KEY;
	    $shop_id=$this->ID;
	    $HTTP = $posted;
        foreach ($HTTP as $Key=>$Value) { $$Key = $Value; }
	    $order = new WC_Order($LMI_PAYMENT_NO);
	    $out_summ = number_format($order->order_total, 2, '.', '');
        if ($out_summ!=$LMI_PAYMENT_AMOUNT) {
	        return false;
	    }
	    if ($LMI_PAYEE_PURSE!=$shop_id) {
	         return false;
	    } 
		if(isset($LMI_SECRET_KEY)) {
     		if($LMI_SECRET_KEY!=$merch_key)  { 
			    return false; 
		    } else {
			    return true;
		    }
	    } else {
            $CalcHash = md5($LMI_PAYEE_PURSE.$LMI_PAYMENT_AMOUNT.$LMI_PAYMENT_NO.$LMI_MODE.$LMI_SYS_INVS_NO.$LMI_SYS_TRANS_NO.$LMI_SYS_TRANS_DATE.$merch_key.$LMI_PAYER_PURSE.$LMI_PAYER_WM);
		    if($LMI_HASH != strtoupper($CalcHash)) {
                return false;
		    } else {
			    return true;
		    }
        }	
	}
	
	/**
	* Check Response
	**/
	
	function check_assistant_response(){

		global $woocommerce;
		$_REQUEST = stripslashes_deep($_REQUEST);
		
		foreach  ($_REQUEST as $key=>$value) {
		    if (strstr($key, '&')  ||strstr ($key, 'amp;') ) {
			    unset ($_REQUEST[$key]);
			    $key=str_replace("&", '', $key);
				$key=str_replace("amp;", '', $key);
				$_REQUEST[$key] = $value;
			}
		}
        
        if (isset($_REQUEST['zpayment']) AND $_REQUEST['zpayment'] == 'callback'){

	            if(isset($_REQUEST['LMI_PREREQUEST']) && $_REQUEST['LMI_PREREQUEST']==1) {
                     die ('YES');
	            } else {
                    if ($this->check_zprequest_is_valid($_REQUEST)) {
					    
                        $order_id = $_REQUEST['LMI_PAYMENT_NO'];
		                $order = new WC_Order($order_id);
	                    $order->update_status('completed', __('Платеж успешно оплачен', 'woocommerce'));
						die('YES');
                    } else {
				        wp_die('FAIL');
			        }
	            }
        }

		else if (isset($_REQUEST['zpayment']) AND $_REQUEST['zpayment'] == 'success'){
			$order_id = $_REQUEST['LMI_PAYMENT_NO'];
			$order = new WC_Order($order_id);
			$woocommerce->cart->empty_cart();
			wp_redirect($this->get_return_url($order));
			exit;

		}
		else if (isset($_REQUEST['zpayment']) AND $_REQUEST['zpayment'] == 'fail'){
			$order_id = $_REQUEST['LMI_PAYMENT_NO'];
			$order = new WC_Order($order_id);
			$order->update_status('failed', __('Платеж не оплачен', 'woocommerce'));
			wp_redirect($order->get_cancel_order_url());
			exit;
		}		
	}

}




/**
 * Add the gateway to WooCommerce
 **/
function add_zpayment_gateway($methods){
	$methods[] = 'WC_Zpayment';
	return $methods;
}

add_filter('woocommerce_payment_gateways', 'add_zpayment_gateway');
}
?>